import 'package:static_postgres_orm/static_postgres_orm.dart';
import 'package:dartz/dartz.dart';
import 'package:data_db/data_db.dart';
import 'package:postgres/postgres.dart';
import 'package:static_postgres_orm/src/domain/orm_filter.dart';

enum _Person { id, company_id, person_id, name }

class _DataFields extends GenericDataFields {
  late final SerialField_PG _id;
  late final IntegerField_PG _company_id;
  late final IntegerField_PG _person_id;
  late final StringField_PG _name;

  _DataFields() : super() {
    fields = <GenericField>[];
    _createFields();
    _addFields();
  }

  _DataFields get _backup => getBackup(this) as _DataFields;
  set _backup(_DataFields v) => setBackup(this, v);

  void _createFields() {
    _id = SerialField_PG(this, 'id', true, true);
    _company_id = IntegerField_PG(this, 'company_id', true, false);
    _person_id = IntegerField_PG(this, 'person_id', true, false);
    _name = StringField_PG(this, 'name', false, false, 100);
  }

  GenericField getCooField(_DataFields origin, String field) {
    return origin.fields
        .firstWhere((element) => getFieldName(element) == field);
  }

  void _cloneField(_DataFields origin) {
    fields.forEach((element) {
      element.copy(getCooField(origin, getFieldName(element)));
    });
  }

  @override
  void backup() {
    _backup = _DataFields();
    _backup._cloneField(this);
  }

  @override
  void restore() {
    if (assigned) {
      _cloneField(_backup);
      finalize();
    }
  }

  void _addFields() {
    fields.add(_id);
    fields.add(_company_id);
    fields.add(_person_id);
    fields.add(_name);
  }
}

class Person_ORM extends PostgressORM {
  static final NewRecordEvent _addRecord = NewRecordEvent();
  _DataFields get _dataFields => getRecords(this)[rowIndex] as _DataFields;

  SerialField_PG get id => _dataFields._id;
  IntegerField_PG get company_id => _dataFields._company_id;
  IntegerField_PG get person_id => _dataFields._person_id;
  StringField_PG get name => _dataFields._name;

  Person_ORM(Postgres_SqlConnection sqlConnection)
      : super(sqlConnection, 'public', 'person', _addRecord) {
    _addRecord.action = _executeAddRecord;
    getRecords(this).add(_DataFields());
  }

  Person_ORM.fromConnection(PostgreSQLExecutionContext _pgConnection)
      : super.fromConnection(_pgConnection, 'public', 'product', _addRecord) {
    _addRecord.action = _executeAddRecord;
    getRecords(this).add(_DataFields());
  }

  Person_ORM.fromDataset(Postgres_SqlConnection sqlConnection, Dataset dataset)
      : super(sqlConnection, 'public', 'person', _addRecord) {
    _addRecord.action = _executeAddRecord;
    getRecords(this).add(_DataFields());
    if (dataset.isNotempty) {
      dataset.first();
      for (var i = 1; i <= dataset.count; i++) {
        newRecord();
        _dataFields.fields.forEach((element) {
          element.loadDataFromDataset(dataset);
        });
        _dataFields.finalize();
        dataset.next();
      }
    }
  }

  Future<Either<ErrorSqlResult, Dataset>> select(
      {Columns? columns,
      Where? where,
      OrderBy? orderBy,
      int? limit,
      int? offset}) async {
    var col = columns;
    col ??= Columns();
    var whe = where;
    whe ??= Where();
    var ordBy = orderBy;
    ordBy ??= OrderBy();
    return (await sqlConnection.openQuery('select',
            getSelectSql(col, whe, ordBy, limit: limit, offset: offset)))
        .fold((l) => left(l), (r) => right(r.dataset));
  }

  Future<Either<ErrorSqlResult, Person_ORM>> getProductWhere(
      OrmSelectWhere where) async {
    return (await sqlConnection.openQuery(
            'select', getSelectSql(Columns(), where, OrderBy())))
        .fold((l) => left(l), (r) {
      return right(Person_ORM.fromDataset(
          sqlConnection as Postgres_SqlConnection, r.dataset));
    });
  }

  GenericDataFields _executeAddRecord() => _DataFields();

  Future<Either<ErrorSqlResult, SelectSuccesSqlResult>> materialize(
      int id) async {
    return getMaterialize(
        this,
        (<GenericField>[])
          ..add(SerialField_PG.clone(_dataFields._id)..setValue(id)));
  }

  Future<Either<ErrorSqlResult, ExecuteSuccesSqlResult>> deleteRecord(
      int id) async {
    return getDeleteRecord(
        this,
        (<GenericField>[])
          ..add(SerialField_PG.clone(_dataFields._id)..setValue(id)));
  }

  Future<Either<ErrorSqlResult, SelectSuccesSqlResult>>
      materializeFromProto3Json(Map<String, dynamic> proto3Json) async {
    if ((proto3Json.containsKey('id'))) {
      return getMaterialize(
          this,
          (<GenericField>[])
            ..add(SerialField_PG.clone(_dataFields._id)
              ..setValue(proto3Json['id'])));
    } else {
      return left(ErrorSqlResult('Product_ORM', 'materializeFromProto3Json', '',
          ['Required fields not found']));
    }
  }
}

class _FilterId extends IntegerFilter {
  _FilterId() : super('person.id');
}

class _FilterCompanyId extends IntegerFilter {
  _FilterCompanyId() : super('person.company_id');
}

class _FilterPersonId extends IntegerFilter {
  _FilterPersonId() : super('person.person_id');
}

class _FilterName extends StringFilter with NullableFilter {
  _FilterName() : super('person.name');
}

class Where extends OrmSelectWhere {
  final _id = _FilterId();
  final _company_id = _FilterCompanyId();
  final _person_id = _FilterPersonId();
  final _name = _FilterName();

  _FilterId get id => _id;
  _FilterCompanyId get company_id => _company_id;
  _FilterPersonId get person_id => _person_id;
  _FilterName get name => _name;

  Where() {
    filters.add(_id);
    filters.add(_company_id);
    filters.add(_person_id);
    filters.add(_name);
  }
}

class Columns implements OrmSelectableColumns {
  final _columns = <_Person>[];
  void get id => _columns.add(_Person.id);
  void get company_id => _columns.add(_Person.company_id);
  void get person_id => _columns.add(_Person.person_id);
  void get name => _columns.add(_Person.name);

  @override
  String toString() {
    List<_Person> list;

    if (_columns.isNotEmpty) {
      list = _columns;
    } else {
      list = _Person.values;
    }

    var col = ' ';
    list.forEach((element) {
      col += ', ' + element.toString().replaceFirst('_', '').toLowerCase();
    });
    return col.replaceFirst(', ', '');
  }
}

class OrderBy with GenericOrderBy {
  void id([ordType = OrdType.asc]) {
    add(ColumnOrdenator('person.id', ordType));
  }

  void company_id([ordType = OrdType.asc]) {
    add(ColumnOrdenator('person.company_id', ordType));
  }

  void person_id([ordType = OrdType.asc]) {
    add(ColumnOrdenator('person.person_id', ordType));
  }

  void name([ordType = OrdType.asc]) {
    add(ColumnOrdenator('person.name', ordType));
  }
}
