import 'package:static_postgres_orm/static_postgres_orm.dart';
import 'package:dartz/dartz.dart';
import 'package:data_db/data_db.dart';
import 'package:postgres/postgres.dart';
import 'package:static_postgres_orm/src/domain/orm_filter.dart';

enum _Product {
  company_id,
  product_id,
  code,
  description,
  is_active,
  price,
  full_description,
  insert_date,
  update_date,
  insert_time,
  category_id
}

class _DataFields extends GenericDataFields {
  late final IntegerField_PG _company_id;
  late final IntegerField_PG _product_id;
  late final StringField_PG _code;
  late final StringField_PG _description;
  late final BooleanField_WithDefault_PG _is_active;
  late final NumericField_WithDefault_PG _price;
  late final StringField_PG _full_description;
  late final DateField_PG _insert_date;
  late final DateTimeField_PG _update_date;
  late final TimeField_PG _insert_time;
  late final IntegerField_PG _category_id;

  _DataFields() : super() {
    fields = <GenericField>[];
    _createFields();
    _addFields();
  }

  _DataFields get _backup => getBackup(this) as _DataFields;
  set _backup(_DataFields v) => setBackup(this, v);

  void _createFields() {
    _company_id = IntegerField_PG(this, 'company_id', true, true);
    _product_id = IntegerField_PG(this, 'product_id', true, true);
    _code = StringField_PG(this, 'code', false, false, 50);
    _description = StringField_PG(this, 'description', false, false, 200);
    _is_active =
        BooleanField_WithDefault_PG(this, 'is_active', false, false, false);
    _price = NumericField_WithDefault_PG(this, 'price', false, false, 0);
    _full_description =
        StringField_PG(this, 'full_description', false, false, -1);
    _insert_date = DateField_PG(this, 'insert_date', false, false);
    _update_date = DateTimeField_PG(this, 'update_date', false, false);
    _insert_time = TimeField_PG(this, 'insert_time', false, false);
    _category_id = IntegerField_PG(this, 'category_id', false, false);
  }

  GenericField getCooField(_DataFields origin, String field) {
    return origin.fields
        .firstWhere((element) => getFieldName(element) == field);
  }

  void _cloneField(_DataFields origin) {
    fields.forEach((element) {
      element.copy(getCooField(origin, getFieldName(element)));
    });
  }

  @override
  void backup() {
    _backup = _DataFields();
    _backup._cloneField(this);
  }

  @override
  void restore() {
    if (assigned) {
      _cloneField(_backup);
      finalize();
    }
  }

  void _addFields() {
    fields.add(_company_id);
    fields.add(_product_id);
    fields.add(_code);
    fields.add(_description);
    fields.add(_is_active);
    fields.add(_price);
    fields.add(_full_description);
    fields.add(_insert_date);
    fields.add(_update_date);
    fields.add(_insert_time);
    fields.add(_category_id);
  }
}

class Product_ORM extends PostgressORM {
  static final NewRecordEvent _addRecord = NewRecordEvent();
  _DataFields get _dataFields => getRecords(this)[rowIndex] as _DataFields;

  IntegerField_PG get company_id => _dataFields._company_id;
  IntegerField_PG get product_id => _dataFields._product_id;
  StringField_PG get code => _dataFields._code;
  StringField_PG get description => _dataFields._description;
  BooleanField_WithDefault_PG get is_active => _dataFields._is_active;
  NumericField_WithDefault_PG get price => _dataFields._price;
  StringField_PG get full_description => _dataFields._full_description;
  DateField_PG get insert_date => _dataFields._insert_date;
  DateTimeField_PG get update_date => _dataFields._update_date;
  TimeField_PG get insert_time => _dataFields._insert_time;
  IntegerField_PG get category_id => _dataFields._category_id;

  Product_ORM(Postgres_SqlConnection sqlConnection)
      : super(sqlConnection, 'public', 'product', _addRecord) {
    _addRecord.action = _executeAddRecord;
    getRecords(this).add(_DataFields());
  }

  Product_ORM.fromConnection(PostgreSQLExecutionContext _pgConnection)
      : super.fromConnection(_pgConnection, 'public', 'product', _addRecord) {
    _addRecord.action = _executeAddRecord;
    getRecords(this).add(_DataFields());
  }

  Product_ORM.fromDataset(Postgres_SqlConnection sqlConnection, Dataset dataset)
      : super(sqlConnection, 'public', 'product', _addRecord) {
    _addRecord.action = _executeAddRecord;
    getRecords(this).add(_DataFields());
    if (dataset.isNotempty) {
      dataset.first();
      for (var i = 1; i <= dataset.count; i++) {
        newRecord();
        _dataFields.fields.forEach((element) {
          element.loadDataFromDataset(dataset);
        });
        _dataFields.finalize();
        dataset.next();
      }
    }
  }

  Future<Either<ErrorSqlResult, Dataset>> select(
      {Columns? columns,
      Where? where,
      OrderBy? orderBy,
      int? limit,
      int? offset}) async {
    var col = columns;
    col ??= Columns();
    var whe = where;
    whe ??= Where();
    var ordBy = orderBy;
    ordBy ??= OrderBy();
    return (await sqlConnection.openQuery('select',
            getSelectSql(col, whe, ordBy, limit: limit, offset: offset)))
        .fold((l) => left(l), (r) => right(r.dataset));
  }

  Future<Either<ErrorSqlResult, Product_ORM>> getProductWhere(
      OrmSelectWhere where) async {
    return (await sqlConnection.openQuery(
            'select', getSelectSql(Columns(), where, OrderBy())))
        .fold((l) => left(l), (r) {
      return right(Product_ORM.fromDataset(
          sqlConnection as Postgres_SqlConnection, r.dataset));
    });
  }

  GenericDataFields _executeAddRecord() => _DataFields();

  Future<Either<ErrorSqlResult, SelectSuccesSqlResult>> materialize(
      int company_id, int product_id) async {
    return getMaterialize(
        this,
        (<GenericField>[])
          ..add(IntegerField_PG.clone(_dataFields._company_id)
            ..setValue(company_id))
          ..add(IntegerField_PG.clone(_dataFields._product_id)
            ..setValue(product_id)));
  }

  Future<Either<ErrorSqlResult, ExecuteSuccesSqlResult>> deleteRecord(
      int company_id, int product_id) async {
    return getDeleteRecord(
        this,
        (<GenericField>[])
          ..add(IntegerField_PG.clone(_dataFields._company_id)
            ..setValue(company_id))
          ..add(IntegerField_PG.clone(_dataFields._product_id)
            ..setValue(product_id)));
  }

  Future<Either<ErrorSqlResult, SelectSuccesSqlResult>>
      materializeFromProto3Json(Map<String, dynamic> proto3Json) async {
    if ((proto3Json.containsKey('companyId')) &&
        (proto3Json.containsKey('productId'))) {
      return getMaterialize(
          this,
          (<GenericField>[])
            ..add(IntegerField_PG.clone(_dataFields._company_id)
              ..setValue(proto3Json['companyId']))
            ..add(IntegerField_PG.clone(_dataFields._product_id)
              ..setValue(proto3Json['productId'])));
    } else {
      return left(ErrorSqlResult('Product_ORM', 'materializeFromProto3Json', '',
          ['Required fields not found']));
    }
  }
}

class _FilterCompanyId extends IntegerFilter {
  _FilterCompanyId() : super('product.company_id');
}

class _FilterProductId extends IntegerFilter {
  _FilterProductId() : super('product.product_id');
}

class _FilterCode extends StringFilter with NullableFilter {
  _FilterCode() : super('product.code');
}

class _FilterDescription extends StringFilter with NullableFilter {
  _FilterDescription() : super('product.description');
}

class _FilterIsActive extends BooleanFilter with NullableFilter {
  _FilterIsActive() : super('product.is_active');
}

class _FilterPrice extends NumericFilter with NullableFilter {
  _FilterPrice() : super('product.price');
}

class _FilterFullDescription extends StringFilter with NullableFilter {
  _FilterFullDescription() : super('product.full_description');
}

class _FilterInsertDate extends DateFilter with NullableFilter {
  _FilterInsertDate() : super('product.insert_date');
}

class _FilterUpdateDate extends DateTimeFilter with NullableFilter {
  _FilterUpdateDate() : super('product.update_date');
}

class _FilterInsertTime extends TimeFilter with NullableFilter {
  _FilterInsertTime() : super('product.insert_time');
}

class _FilterCategoryId extends IntegerFilter with NullableFilter {
  _FilterCategoryId() : super('product.category_id');
}

class Where extends OrmSelectWhere {
  final _company_id = _FilterCompanyId();
  final _product_id = _FilterProductId();
  final _code = _FilterCode();
  final _description = _FilterDescription();
  final _is_active = _FilterIsActive();
  final _price = _FilterPrice();
  final _full_description = _FilterFullDescription();
  final _insert_date = _FilterInsertDate();
  final _update_date = _FilterUpdateDate();
  final _insert_time = _FilterInsertTime();
  final _category_id = _FilterCategoryId();

  _FilterCompanyId get company_id => _company_id;
  _FilterProductId get product_id => _product_id;
  _FilterCode get code => _code;
  _FilterDescription get description => _description;
  _FilterIsActive get is_active => _is_active;
  _FilterPrice get price => _price;
  _FilterFullDescription get full_description => _full_description;
  _FilterInsertDate get insert_date => _insert_date;
  _FilterUpdateDate get update_date => _update_date;
  _FilterInsertTime get insert_time => _insert_time;
  _FilterCategoryId get category_id => _category_id;

  Where() {
    filters.add(_company_id);
    filters.add(_product_id);
    filters.add(_code);
    filters.add(_description);
    filters.add(_is_active);
    filters.add(_price);
    filters.add(_full_description);
    filters.add(_insert_date);
    filters.add(_update_date);
    filters.add(_insert_time);
    filters.add(_category_id);
  }
}

class Columns implements OrmSelectableColumns {
  final _columns = <_Product>[];
  void get company_id => _columns.add(_Product.company_id);
  void get product_id => _columns.add(_Product.product_id);
  void get code => _columns.add(_Product.code);
  void get description => _columns.add(_Product.description);
  void get is_active => _columns.add(_Product.is_active);
  void get price => _columns.add(_Product.price);
  void get full_description => _columns.add(_Product.full_description);
  void get insert_date => _columns.add(_Product.insert_date);
  void get update_date => _columns.add(_Product.update_date);
  void get insert_time => _columns.add(_Product.insert_time);
  void get category_id => _columns.add(_Product.category_id);

  @override
  String toString() {
    List<_Product> list;

    if (_columns.isNotEmpty) {
      list = _columns;
    } else {
      list = _Product.values;
    }

    var col = ' ';
    list.forEach((element) {
      col += ', ' + element.toString().replaceFirst('_', '').toLowerCase();
    });
    return col.replaceFirst(', ', '');
  }
}

class OrderBy with GenericOrderBy {
  void company_id([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.company_id', ordType));
  }

  void product_id([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.product_id', ordType));
  }

  void code([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.code', ordType));
  }

  void description([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.description', ordType));
  }

  void is_active([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.is_active', ordType));
  }

  void price([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.price', ordType));
  }

  void full_description([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.full_description', ordType));
  }

  void insert_date([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.insert_date', ordType));
  }

  void update_date([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.update_date', ordType));
  }

  void insert_time([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.insert_time', ordType));
  }

  void category_id([ordType = OrdType.asc]) {
    add(ColumnOrdenator('product.category_id', ordType));
  }
}
