import 'package:static_postgres_orm/src/domain/orm_filter_operator.dart';
import 'package:test/test.dart';

void main() async {
  test('Null', () {
    expect(IsNullOperator('code').getSqlSyntax(), ' and code is null');
    expect(IsNullOperator('code', LogicalOperator.none).getSqlSyntax(),
        'code is null');
    expect(IsNullOperator('code', LogicalOperator.or).getSqlSyntax(),
        ' or code is null');

    expect(IsNotNullOperator('code').getSqlSyntax(), ' and code is not null');
    expect(IsNotNullOperator('code', LogicalOperator.none).getSqlSyntax(),
        'code is not null');
  });

  group('String', () {
    test('EqualsTo', () {
      expect(StringOperatorEqualsTo('code', 'teste').getSqlSyntax(),
          " and code = 'teste'");
      expect(
          StringOperatorEqualsTo('code', 'teste', LogicalOperator.none)
              .getSqlSyntax(),
          "code = 'teste'");
      expect(
          StringOperatorEqualsTo.fromMap(
                  (StringOperatorEqualsTo('code', 'teste').toMap()))
              .getSqlSyntax(),
          " and code = 'teste'");
    });

    test('NotEqualsTo', () {
      expect(StringOperatorNotEqualsTo('code', 'teste').getSqlSyntax(),
          " and code <> 'teste'");
      expect(
          StringOperatorNotEqualsTo('code', 'teste', LogicalOperator.none)
              .getSqlSyntax(),
          "code <> 'teste'");

      expect(
          StringOperatorNotEqualsTo.fromMap(
                  (StringOperatorNotEqualsTo('code', 'teste').toMap()))
              .getSqlSyntax(),
          " and code <> 'teste'");
    });

    test('In', () {
      expect(StringOperatorIn('code', ['teste1', 'teste2']).getSqlSyntax(),
          " and code in ('teste1', 'teste2')");

      expect(
          StringOperatorIn('code', ['teste1', 'teste2'], LogicalOperator.none)
              .getSqlSyntax(),
          "code in ('teste1', 'teste2')");

      expect(
          StringOperatorIn.fromMap(
                  (StringOperatorIn('code', ['teste1', 'teste2']).toMap()))
              .getSqlSyntax(),
          " and code in ('teste1', 'teste2')");

      expect(StringOperatorIn('code', ['teste1']).getSqlSyntax(),
          " and code in ('teste1')");
      expect(
          StringOperatorIn('code', ['teste1'], LogicalOperator.none)
              .getSqlSyntax(),
          "code in ('teste1')");
    });

    test('NotIn', () {
      expect(StringOperatorNotIn('code', ['teste1', 'teste2']).getSqlSyntax(),
          " and code not in ('teste1', 'teste2')");
      expect(
          StringOperatorNotIn.fromMap(
                  (StringOperatorNotIn('code', ['teste1', 'teste2']).toMap()))
              .getSqlSyntax(),
          " and code not in ('teste1', 'teste2')");
      expect(
          StringOperatorNotIn(
                  'code', ['teste1', 'teste2'], LogicalOperator.none)
              .getSqlSyntax(),
          "code not in ('teste1', 'teste2')");
      expect(StringOperatorNotIn('code', ['teste1']).getSqlSyntax(),
          " and code not in ('teste1')");
      expect(
          StringOperatorNotIn('code', ['teste1'], LogicalOperator.none)
              .getSqlSyntax(),
          "code not in ('teste1')");
    });

    test('BeginsWith', () {
      expect(StringOperatorBeginsWith('code', 't').getSqlSyntax(),
          " and code like 't%'");
      expect(
          StringOperatorBeginsWith('code', 't', LogicalOperator.none)
              .getSqlSyntax(),
          "code like 't%'");
      expect(
          StringOperatorBeginsWith.fromMap(
                  (StringOperatorBeginsWith('code', 't').toMap()))
              .getSqlSyntax(),
          " and code like 't%'");
    });

    test('NotBeginsWith', () {
      expect(StringOperatorNotBeginsWith('code', 't').getSqlSyntax(),
          " and code not like 't%'");
      expect(
          StringOperatorNotBeginsWith('code', 't', LogicalOperator.none)
              .getSqlSyntax(),
          "code not like 't%'");
      expect(
          StringOperatorNotBeginsWith.fromMap(
                  (StringOperatorNotBeginsWith('code', 't').toMap()))
              .getSqlSyntax(),
          " and code not like 't%'");
    });

    test('Contais', () {
      expect(StringOperatorContains('code', 't').getSqlSyntax(),
          " and code like '%t%'");
      expect(
          StringOperatorContains('code', 't', LogicalOperator.none)
              .getSqlSyntax(),
          "code like '%t%'");
      expect(
          StringOperatorContains.fromMap(
                  (StringOperatorContains('code', 't').toMap()))
              .getSqlSyntax(),
          " and code like '%t%'");
    });

    test('NotContais', () {
      expect(StringOperatorNotContains('code', 't').getSqlSyntax(),
          " and code not like '%t%'");
      expect(
          StringOperatorNotContains('code', 't', LogicalOperator.none)
              .getSqlSyntax(),
          "code not like '%t%'");
      expect(
          StringOperatorNotContains.fromMap(
                  (StringOperatorNotContains('code', 't').toMap()))
              .getSqlSyntax(),
          " and code not like '%t%'");
    });

    test('EndsWith', () {
      var filter = StringOperatorEndsWith('code', 't');
      expect(filter.getSqlSyntax(), " and code like '%t'");
      expect(StringOperatorEndsWith.fromMap(filter.toMap()).getSqlSyntax(),
          " and code like '%t'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code like '%t'");
    });

    test('NotEndsWith', () {
      var filter = StringOperatorNotEndsWith('code', 't');
      expect(filter.getSqlSyntax(),
          StringOperatorNotEndsWith.fromMap(filter.toMap()).getSqlSyntax());
      expect(filter.getSqlSyntax(), " and code not like '%t'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code not like '%t'");
    });
  });

  group('Integer', () {
    test('EqualsTo', () {
      var filter = IntegerOperatorEqualsTo('code', 1);
      expect(filter.getSqlSyntax(), ' and code = 1');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code = 1');
    });

    test('NotEqualsTo', () {
      var filter = IntegerOperatorNotEqualsTo('code', 1);
      expect(filter.getSqlSyntax(), ' and code <> 1');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code <> 1');
    });

    test('In', () {
      var filter = IntegerOperatorIn('code', [1, 2]);
      expect(filter.getSqlSyntax(), ' and code in (1, 2)');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code in (1, 2)');
      filter = IntegerOperatorIn('code', [1]);
      expect(filter.getSqlSyntax(), ' and code in (1)');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code in (1)');
    });

    test('NotIn', () {
      var filter = IntegerOperatorNotIn('code', [1, 2]);
      expect(filter.getSqlSyntax(), ' and code not in (1, 2)');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code not in (1, 2)');
      filter = IntegerOperatorNotIn('code', [1]);
      expect(filter.getSqlSyntax(), ' and code not in (1)');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code not in (1)');
    });

    test('Between', () {
      var filter = IntegerOperatorBetween('code', 1, 2);
      expect(filter.getSqlSyntax(), ' and code between 1 and 2');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code between 1 and 2');
    });

    test('Not Between', () {
      var filter = IntegerOperatorNotBetween('code', 1, 2);
      expect(filter.getSqlSyntax(), ' and code not between 1 and 2');
      expect(filter.getSqlSyntax(LogicalOperator.none),
          'code not between 1 and 2');
    });

    test('MoreThan', () {
      var filter = IntegerOperatorMoreThan('code', 1);
      expect(filter.getSqlSyntax(), ' and code > 1');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code > 1');
      filter = IntegerOperatorMoreThan('code', 1, true);
      expect(filter.getSqlSyntax(), ' and code >= 1');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code >= 1');
    });

    test('LessThan', () {
      var filter = IntegerOperatorLessThan('code', 1);
      expect(filter.getSqlSyntax(), ' and code < 1');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code < 1');
      filter = IntegerOperatorLessThan('code', 1, true);
      expect(filter.getSqlSyntax(), ' and code <= 1');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code <= 1');
    });
  });

  group('Numeric', () {
    test('EqualsTo', () {
      var filter = NumericOperatorEqualsTo('code', 1.89);
      expect(filter.getSqlSyntax(), ' and code = 1.89');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code = 1.89');
    });

    test('NotEqualsTo', () {
      var filter = NumericOperatorNotEqualsTo('code', 1.89);
      expect(filter.getSqlSyntax(), ' and code <> 1.89');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code <> 1.89');
    });

    test('In', () {
      var filter = NumericOperatorIn('code', [1.89, 2.89]);
      expect(filter.getSqlSyntax(), ' and code in (1.89, 2.89)');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code in (1.89, 2.89)');
      filter = NumericOperatorIn('code', [1.89]);
      expect(filter.getSqlSyntax(), ' and code in (1.89)');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code in (1.89)');
    });

    test('NotIn', () {
      var filter = NumericOperatorNotIn('code', [1.89, 2.89]);
      expect(filter.getSqlSyntax(), ' and code not in (1.89, 2.89)');
      expect(filter.getSqlSyntax(LogicalOperator.none),
          'code not in (1.89, 2.89)');
      filter = NumericOperatorNotIn('code', [1.89]);
      expect(filter.getSqlSyntax(), ' and code not in (1.89)');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code not in (1.89)');
    });

    test('Between', () {
      var filter = NumericOperatorBetween('code', 1.89, 2.89);
      expect(filter.getSqlSyntax(), ' and code between 1.89 and 2.89');
      expect(filter.getSqlSyntax(LogicalOperator.none),
          'code between 1.89 and 2.89');
    });

    test('NotBetween', () {
      var filter = NumericOperatorNotBetween('code', 1.89, 2.89);
      expect(filter.getSqlSyntax(), ' and code not between 1.89 and 2.89');
      expect(filter.getSqlSyntax(LogicalOperator.none),
          'code not between 1.89 and 2.89');
    });

    test('MoreThan', () {
      var filter = NumericOperatorMoreThan('code', 1.89);
      expect(filter.getSqlSyntax(), ' and code > 1.89');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code > 1.89');
      filter = NumericOperatorMoreThan('code', 1.89, true);
      expect(filter.getSqlSyntax(), ' and code >= 1.89');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code >= 1.89');
    });

    test('LessThan', () {
      var filter = NumericOperatorLessThan('code', 1.89);
      expect(filter.getSqlSyntax(), ' and code < 1.89');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code < 1.89');
      filter = NumericOperatorLessThan('code', 1.89, true);
      expect(filter.getSqlSyntax(), ' and code <= 1.89');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code <= 1.89');
    });
  });

  group('Boolean', () {
    test('IsTrue', () {
      var filter = OperatorIsTrue('code');
      expect(filter.getSqlSyntax(), ' and code is true');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code is true');
    });

    test('NotIsTrue', () {
      var filter = OperatorNotIsTrue('code');
      expect(filter.getSqlSyntax(), ' and code is not true');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code is not true');
    });

    test('IsFalse', () {
      var filter = OperatorIsFalse('code');
      expect(filter.getSqlSyntax(), ' and code is false');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code is false');
    });

    test('NotIsFalse', () {
      var filter = OperatorNotIsFalse('code');
      expect(filter.getSqlSyntax(), ' and code is not false');
      expect(filter.getSqlSyntax(LogicalOperator.none), 'code is not false');
    });
  });

  group('Date', () {
    test('EqualsTo', () {
      var filter =
          DateOperatorEqualsTo('code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code = '2019-09-12'");
      expect(filter.getSqlSyntax(),
          DateOperatorEqualsTo.fromMap(filter.toMap()).getSqlSyntax());
      expect(filter.getSqlSyntax(LogicalOperator.none), "code = '2019-09-12'");
    });

    test('NotEqualsTo', () {
      var filter = DateOperatorNotEqualsTo('code', DateTime(2019, 09, 12));
      expect(filter.getSqlSyntax(), " and code <> '2019-09-12'");
      expect(filter.getSqlSyntax(),
          DateOperatorNotEqualsTo.fromMap(filter.toMap()).getSqlSyntax());
      expect(filter.getSqlSyntax(LogicalOperator.none), "code <> '2019-09-12'");
    });

    test('In', () {
      var filter = DateOperatorIn(
          'code', [DateTime(2019, 09, 12), DateTime(2020, 09, 12)]);
      expect(
          filter.getSqlSyntax(), " and code in ('2019-09-12', '2020-09-12')");
      expect(filter.getSqlSyntax(),
          DateOperatorIn.fromMap(filter.toMap()).getSqlSyntax());
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code in ('2019-09-12', '2020-09-12')");
    });

    test('NotIn', () {
      var filter = DateOperatorNotIn(
          'code', [DateTime(2019, 09, 12), DateTime(2020, 09, 12)]);
      expect(filter.getSqlSyntax(),
          " and code not in ('2019-09-12', '2020-09-12')");
      expect(filter.getSqlSyntax(),
          DateOperatorNotIn.fromMap(filter.toMap()).getSqlSyntax());
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code not in ('2019-09-12', '2020-09-12')");
    });

    test('Between', () {
      var filter = DateOperatorBetween(
          'code', DateTime(2019, 09, 12), DateTime(2020, 09, 12));
      expect(filter.getSqlSyntax(),
          " and code between '2019-09-12' and '2020-09-12'");
      expect(filter.getSqlSyntax(),
          DateOperatorBetween.fromMap(filter.toMap()).getSqlSyntax());
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code between '2019-09-12' and '2020-09-12'");
    });

    test('NotBetween', () {
      var filter = DateOperatorNotBetween(
          'code', DateTime(2019, 09, 12), DateTime(2020, 09, 12));
      expect(filter.getSqlSyntax(),
          " and code not between '2019-09-12' and '2020-09-12'");
      expect(filter.getSqlSyntax(),
          DateOperatorNotBetween.fromMap(filter.toMap()).getSqlSyntax());
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code not between '2019-09-12' and '2020-09-12'");
    });

    test('MoreThan', () {
      var filter = DateOperatorMoreThan('code', DateTime(2019, 09, 12));
      expect(filter.getSqlSyntax(), " and code > '2019-09-12'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code > '2019-09-12'");
      expect(filter.getSqlSyntax(),
          DateOperatorMoreThan.fromMap(filter.toMap()).getSqlSyntax());
      filter = DateOperatorMoreThan('code', DateTime(2019, 09, 12), true);
      expect(filter.getSqlSyntax(), " and code >= '2019-09-12'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code >= '2019-09-12'");
    });

    test('LessThan', () {
      var filter = DateOperatorLessThan('code', DateTime(2019, 09, 12));
      expect(filter.getSqlSyntax(), " and code < '2019-09-12'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code < '2019-09-12'");
      expect(filter.getSqlSyntax(),
          DateOperatorLessThan.fromMap(filter.toMap()).getSqlSyntax());
      filter = DateOperatorLessThan('code', DateTime(2019, 09, 12), true);
      expect(filter.getSqlSyntax(), " and code <= '2019-09-12'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code <= '2019-09-12'");
    });
  });

  group('Date Time', () {
    test('EqualsTo', () {
      var filter =
          DateTimeOperatorEqualsTo('code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code = '2019-09-12 08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code = '2019-09-12 08:10:15'");
    });

    test('NotEqualsTo', () {
      var filter = DateTimeOperatorNotEqualsTo(
          'code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code <> '2019-09-12 08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code <> '2019-09-12 08:10:15'");
    });

    test('In', () {
      var filter = DateTimeOperatorIn('code', [
        DateTime(2019, 09, 12, 8, 10, 15),
        DateTime(2020, 09, 12, 8, 10, 15)
      ]);
      expect(filter.getSqlSyntax(),
          " and code in ('2019-09-12 08:10:15', '2020-09-12 08:10:15')");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code in ('2019-09-12 08:10:15', '2020-09-12 08:10:15')");
    });

    test('NotIn', () {
      var filter = DateTimeOperatorNotIn('code', [
        DateTime(2019, 09, 12, 8, 10, 15),
        DateTime(2020, 09, 12, 8, 10, 15)
      ]);
      expect(filter.getSqlSyntax(),
          " and code not in ('2019-09-12 08:10:15', '2020-09-12 08:10:15')");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code not in ('2019-09-12 08:10:15', '2020-09-12 08:10:15')");
    });

    test('Between', () {
      var filter = DateTimeOperatorBetween('code',
          DateTime(2019, 09, 12, 8, 10, 15), DateTime(2020, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(),
          " and code between '2019-09-12 08:10:15' and '2020-09-12 08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code between '2019-09-12 08:10:15' and '2020-09-12 08:10:15'");
    });

    test('NotBetween', () {
      var filter = DateTimeOperatorNotBetween('code',
          DateTime(2019, 09, 12, 8, 10, 15), DateTime(2020, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(),
          " and code not between '2019-09-12 08:10:15' and '2020-09-12 08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code not between '2019-09-12 08:10:15' and '2020-09-12 08:10:15'");
    });

    test('MoreThan', () {
      var filter =
          DateTimeOperatorMoreThan('code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code > '2019-09-12 08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code > '2019-09-12 08:10:15'");
      filter = DateTimeOperatorMoreThan(
          'code', DateTime(2019, 09, 12, 8, 10, 15), true);
      expect(filter.getSqlSyntax(), " and code >= '2019-09-12 08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code >= '2019-09-12 08:10:15'");
    });

    test('LessThan', () {
      var filter =
          DateTimeOperatorLessThan('code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code < '2019-09-12 08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code < '2019-09-12 08:10:15'");
      filter = DateTimeOperatorLessThan(
          'code', DateTime(2019, 09, 12, 8, 10, 15), true);
      expect(filter.getSqlSyntax(), " and code <= '2019-09-12 08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code <= '2019-09-12 08:10:15'");
    });
  });

  group('Time', () {
    test('EqualsTo', () {
      var filter =
          TimeOperatorEqualsTo('code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code = '08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code = '08:10:15'");
    });

    test('NotEqualsTo', () {
      var filter =
          TimeOperatorNotEqualsTo('code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code <> '08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code <> '08:10:15'");
    });

    test('In', () {
      var filter = TimeOperatorIn('code', [
        DateTime(2019, 09, 12, 8, 10, 15),
        DateTime(2020, 09, 12, 8, 10, 15)
      ]);
      expect(filter.getSqlSyntax(), " and code in ('08:10:15', '08:10:15')");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code in ('08:10:15', '08:10:15')");
    });

    test('NotIn', () {
      var filter = TimeOperatorNotIn('code', [
        DateTime(2019, 09, 12, 8, 10, 15),
        DateTime(2020, 09, 12, 8, 10, 15)
      ]);
      expect(
          filter.getSqlSyntax(), " and code not in ('08:10:15', '08:10:15')");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code not in ('08:10:15', '08:10:15')");
    });

    test('Between', () {
      var filter = TimeOperatorBetween('code',
          DateTime(2019, 09, 12, 8, 10, 15), DateTime(2020, 09, 12, 8, 10, 15));
      expect(
          filter.getSqlSyntax(), " and code between '08:10:15' and '08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code between '08:10:15' and '08:10:15'");
    });

    test('NotBetween', () {
      var filter = TimeOperatorNotBetween('code',
          DateTime(2019, 09, 12, 8, 10, 15), DateTime(2020, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(),
          " and code not between '08:10:15' and '08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none),
          "code not between '08:10:15' and '08:10:15'");
    });

    test('MoreThan', () {
      var filter =
          TimeOperatorMoreThan('code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code > '08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code > '08:10:15'");
      filter =
          TimeOperatorMoreThan('code', DateTime(2019, 09, 12, 8, 10, 15), true);
      expect(filter.getSqlSyntax(), " and code >= '08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code >= '08:10:15'");
    });

    test('LessThan', () {
      var filter =
          TimeOperatorLessThan('code', DateTime(2019, 09, 12, 8, 10, 15));
      expect(filter.getSqlSyntax(), " and code < '08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code < '08:10:15'");
      filter =
          TimeOperatorLessThan('code', DateTime(2019, 09, 12, 8, 10, 15), true);
      expect(filter.getSqlSyntax(), " and code <= '08:10:15'");
      expect(filter.getSqlSyntax(LogicalOperator.none), "code <= '08:10:15'");
    });
  });
}
