import 'package:test/test.dart';
import 'package:static_postgres_orm/src/domain/orm_filter.dart';

void main() async {
  group('String', () {
    test('loadMap and toMap', () async {
      var x = Code()
        ..beginsWith('1')
        ..contains('2')
        ..endsWith('3')
        ..equalsTo('4')
        ..isIn(['5', '5'])
        ..isNotNull()
        ..isNull()
        ..notBeginsWith('6')
        ..notContains('7')
        ..notEndsWith('8')
        ..notEqualsTo('9')
        ..notIn(['10', '10']);

      var y = Code()..loadMap(x.toMap());
      expect(x.getFilter(), y.getFilter());
    });
  });

  group('Integer', () {
    test('loadMap and toMap', () async {
      var x = ID()
        ..equalsTo(1)
        ..notEqualsTo(2)
        ..isIn([1, 2])
        ..notIn([1, 2])
        ..between(1, 2)
        ..notBetween(1, 2)
        ..moreThan(1)
        ..lessThan(2)
        ..isNotNull()
        ..isNotNull();

      var y = ID()..loadMap(x.toMap());
      expect(x.getFilter(), y.getFilter());
    });
  });

  group('Numeric', () {
    test('loadMap and toMap', () async {
      var x = Price()
        ..equalsTo(2)
        ..notEqualsTo(2)
        ..isIn([1, 2])
        ..notIn([1, 2])
        ..between(1, 2)
        ..notBetween(1, 2)
        ..moreThan(1)
        ..lessThan(2)
        ..isNotNull()
        ..isNotNull();

      var y = Price()..loadMap(x.toMap());
      expect(y.getFilter(), x.getFilter());
    });
  });

  group('DateFilter', () {
    test('loadMap and toMap', () async {
      var x = LastDate()
        ..equalsTo(DateTime.now())
        ..notEqualsTo(DateTime.now())
        ..isIn([DateTime.now(), DateTime.now()])
        ..notIn([DateTime.now(), DateTime.now()])
        ..between(DateTime.now(), DateTime.now())
        ..notBetween(DateTime.now(), DateTime.now())
        ..moreThan(DateTime.now())
        ..lessThan(DateTime.now())
        ..isNotNull()
        ..isNotNull();

      var y = LastDate()..loadMap(x.toMap());
      expect(y.getFilter(), x.getFilter());
    });
  });

  group('TimeFilter', () {
    test('loadMap and toMap', () async {
      var x = LastTime()
        ..equalsTo(DateTime.now())
        ..notEqualsTo(DateTime.now())
        ..isIn([DateTime.now(), DateTime.now()])
        ..notIn([DateTime.now(), DateTime.now()])
        ..between(DateTime.now(), DateTime.now())
        ..notBetween(DateTime.now(), DateTime.now())
        ..moreThan(DateTime.now())
        ..lessThan(DateTime.now())
        ..isNotNull()
        ..isNotNull();

      var y = LastTime()..loadMap(x.toMap());
      expect(y.getFilter(), x.getFilter());
    });
  });

  group('BooleanFilter', () {
    test('loadMap and toMap', () async {
      var x = IsActive()
        ..isTrue()
        ..notIsTrue()
        ..isFalse()
        ..notIsFalse()
        ..isNotNull()
        ..isNotNull();

      var y = IsActive()..loadMap(x.toMap());
      expect(y.getFilter(), x.getFilter());
    });
  });
}

class Code extends StringFilter with NullableFilter {
  Code() : super('code');
}

class ID extends IntegerFilter with NullableFilter {
  ID() : super('id');
}

class Price extends NumericFilter with NullableFilter {
  Price() : super('price');
}

class LastDate extends DateFilter with NullableFilter {
  LastDate() : super('last_date');
}

class FirstDate extends DateTimeFilter with NullableFilter {
  FirstDate() : super('first_date');
}

class LastTime extends TimeFilter with NullableFilter {
  LastTime() : super('last_time');
}

class IsActive extends BooleanFilter with NullableFilter {
  IsActive() : super('is_active');
}
