import 'package:test/test.dart';
import '../example/orm_person_example.dart';
import 'package:static_postgres_orm/static_postgres_orm.dart';

void main() async {
  var _postgresConnection = Postgres_SqlConnection(
      '192.168.1.81', 5432, 'db_teste', 'postgres', '1489');
  _postgresConnection.className = 'tester';

  group('Person_ORM', () {
    setUp(() async {
      await _postgresConnection.executeCommand(
          'delete from person', 'delete from person');
      await _postgresConnection.executeCommand(
          'delete from person', 'ALTER SEQUENCE person_id_seq RESTART WITH 1');
    });

    group('Empty Database', () {
      test('select', () async {
        var orm = Person_ORM(_postgresConnection);
        var isEmpty = false;

        var r = (await orm.select());

        r.map((r) => isEmpty = r.isEmpty);

        expect(r.isRight(), true, reason: 'select must be ok');
        expect(isEmpty, true, reason: 'dataset must be empty');
      });

      test('materialize', () async {
        var orm = Person_ORM(_postgresConnection);
        var r = (await orm.materialize(1));
        var m = '';

        r.leftMap((l) {
          m = l.errorMessage.first;
        });
        expect(r.isRight(), false, reason: 'materialize must not be ok');
        expect(m, 'record not found', reason: 'error message');
      });

      test('deleteRecord', () async {
        var orm = Person_ORM(_postgresConnection);
        var r = (await orm.deleteRecord(1));
        var m = '';

        r.leftMap((l) {
          m = l.errorMessage.first;
        });
        expect(r.isRight(), false, reason: 'deleteRecord must not be ok');
        expect(m, 'record not found', reason: 'error message');
      });

      test('delete', () async {
        var orm = Person_ORM(_postgresConnection);
        orm.id.setValue(1);
        var r = (await orm.delete());
        var m = '';

        r.leftMap((l) {
          m = l.errorMessage.first;
        });
        expect(r.isRight(), false, reason: 'delete must not be ok');
        expect(m, 'record not found', reason: 'error message');
      });

      test('update', () async {
        var orm = Person_ORM(_postgresConnection);
        orm.id.setValue(1);
        orm.company_id.setValue(1);
        orm.person_id.setValue(1);
        var r = (await orm.update());
        var m = '';
        r.leftMap((l) {
          m = l.errorMessage.first;
        });
        expect(r.isRight(), false, reason: 'update must not be ok');
        expect(m, 'record not found', reason: 'error message');
      });

      test('insert', () async {
        var orm = Person_ORM(_postgresConnection);
        orm.company_id.setValue(1);
        orm.person_id.setValue(1);

        var r = (await orm.insert());

        var m = 0;
        orm.id.getValue().map((a) => m = a);
        expect(r.isRight(), true, reason: 'insert must not be ok');
        expect(m == 1, true, reason: 'id must be a sequence value');
      });
    });

    group('Filled Database', () {
      setUp(() async {
        var x = Person_ORM(_postgresConnection)
          ..company_id.setValue(1)
          ..person_id.setValue(1);
        await x.insert();

        x.clear();
        x.company_id.setValue(1);
        x.person_id.setValue(2);

        await x.insert();
      });

      test('select', () async {
        var orm = Person_ORM(_postgresConnection);
        var count = 0;

        var r = (await orm.select(orderBy: OrderBy()..person_id()));

        r.map((dataset) {
          count = dataset.count;
        });

        expect(r.isRight(), true, reason: 'select must be ok');
        expect(count, 2, reason: '2 records are expected');
      });

      test('materialize', () async {
        var orm = Person_ORM(_postgresConnection);
        var r = (await orm.materialize(1));
        expect(r.isRight(), true, reason: 'materialize must be ok');
      });

      test('update', () async {
        var orm = Person_ORM(_postgresConnection);
        await orm.materialize(1);
        orm.name.setValue('andriws');
        var r = (await orm.update());
        expect(r.isRight(), true, reason: 'update must be ok');
      });

      test('deleteRecord', () async {
        var orm = Person_ORM(_postgresConnection);
        var r = (await orm.deleteRecord(1));
        expect(r.isRight(), true, reason: 'deleteRecord must be ok');
      });

      test('delete', () async {
        var orm = Person_ORM(_postgresConnection);
        orm.id.setValue(2);
        var r = (await orm.delete());
        expect(r.isRight(), true, reason: 'delete must be ok');
      });

      test('insert', () async {
        var orm = Person_ORM(_postgresConnection);
        orm.company_id.setValue(1);
        orm.person_id.setValue(3);

        var r = (await orm.insert());

        var m = 0;
        orm.id.getValue().map((a) => m = a);
        expect(r.isRight(), true, reason: 'insert must be ok');
        expect(m == 3, true, reason: 'id must be a sequence value');
      });
    });

    group('GRPC proto3 ', () {
      test('-insertFromProto3Json', () async {
        var orm = Person_ORM(_postgresConnection);
        var x = (await orm.insertFromProto3Json({
          'companyId': {'value': 1},
          'personId': {'value': 1}
        }));
        expect(x.isRight(), true);
      });

      test('-insertFromProto3Json skipping sequence', () async {
        var orm = Person_ORM(_postgresConnection);
        var x = (await orm.insertFromProto3Json({
          'id': {'value': 80},
          'companyId': {'value': 1},
          'personId': {'value': 1}
        }));

        var y = 0;

        orm.id.getValue().map((a) => y = a);
        expect(x.isRight(), true);
        expect(y, 80);
      });

      test('-deleteFromProto3Json', () async {
        var orm = Person_ORM(_postgresConnection);
        orm.person_id.setValue(1);
        orm.company_id.setValue(1);
        await (orm.insert());
        var v = 0;
        orm.id.value.map((a) => v = a);
        var x = (await orm.deleteFromProto3Json({
          'id': {'value': v}
        }));
        expect(x.isRight(), true);
      });

      test('updateFromProto3Json', () async {
        var orm = Person_ORM(_postgresConnection);
        orm.person_id.setValue(1);
        orm.company_id.setValue(1);
        await orm.insert();
        var v = 0;
        orm.id.value.map((a) => v = a);
        orm.clear();
        (await orm.updateFromProto3Json({
          'id': {'value': v},
          'name': {'value': 'updated'}
        }))
            .leftMap((l) => print(l.toJson()));
        var x = '';

        orm.name.value.map((a) => x = a);
        expect(x, 'updated');
        var y = (await orm.updateFromProto3Json({
          'id': {'value': -1},
          'name': {'value': 'updated'}
        }))
            .isRight();
        expect(y, false);
        expect(orm.count, 0);
      });

      test('updateFromProto3Json -invalid', () async {
        var orm = Person_ORM(_postgresConnection);
        orm.person_id.setValue(1);
        orm.company_id.setValue(1);
        await orm.insert();
        orm.clear();
        (await orm.updateFromProto3Json({
          'companyId': {'value': 1},
        }))
            .leftMap((l) => null);
        var x = '';

        orm.name.value.map((a) => x = a);
        expect(x, '');
      });
    });
  });
}
